function areaRmv = AreaRmvCheck(sol_ana, sol_inp_ana, maskTest, energy_ana, ...
        gamma_alpha_min)
%--------------------------------------------------------------------------
%   This program is to find an credible interval for a specified area or 
%       an indivial pixel
%    
%   Usage: areaRmv = AreaRmvCheck(sol_ana, sol_inp_ana, maskTest, ...
%       energy_ana, gamma_alpha_min)
%
%   Inputs: 
%       - sol_ana: optimal solution of given minimisation problem
%       - sol_inp_ana: inpainting result according to given mask
%       - maskTest: given mask represents lost infomation area
%       - energy_ana: operator to compute energy of given problem
%       - gamma_alpha_min: energy upper bound 
%
%   Outputs: 
%       - areaRmv_ana.rmvIdx: 0 - can not be removed, otherwise reture 1
%       - areaRmv_ana.eng_sgt: energy after removing a given area
% 
%   Code by: Xiaohao Cai
%   Last updated: 02/10/2016 
%--------------------------------------------------------------------------

for i=1:length(maskTest)
    % generate surrogate image
    sgt = sol_ana.*(1 - maskTest{i}) +  sol_inp_ana.*maskTest{i};
    
    engSgt = energy_ana(sgt);

    if engSgt <= gamma_alpha_min
        rmvIdx = 1;
    %     disp('This area is removable!');
    else
        rmvIdx = 0;
    %     disp('This area is nonremovable!');
    end
    
    areaRmv{i}.engSgt = engSgt;
    areaRmv{i}.rmvIdx = rmvIdx;
end

