function [bound_l, bound_h] = FindItv4AreaBiLevel(sol,areas,stepSize, ...
    energyG, gamma_alpha_min)
%--------------------------------------------------------------------------
%   This program is to find an credible interval for a specified area or 
%       an indivial pixel, using bilevel strategy
%    
%   Usage: [bound_b, bound_u] = FindItv4AreaBiLevel(sol,areas,stepSize, ...
%        energyG, gamma_alpha_min);
%
%   Inputs: 
%       - sol_ana: optimal solution of given minimisation problem
%       - areas: given area of interest, and find an credible region for it
%       - stepSize: step size used to search credible region
%       - energyG: operator to compute energy of given problem
%       - gamma_alpha_min: energy upper bound 
%
%   Outputs: 
%       - bound_l: lower bound computed
%       - bound_h: upper bound computed
% 
%   Code by: Xiaohao Cai
%   Last updated: 30/10/2017
%--------------------------------------------------------------------------

dummy = sol;

% given area
xIdxL = areas.xIdxL; xIdxH = areas.xIdxH; 
yIdxL = areas.yIdxL; yIdxH = areas.yIdxH;

% compute the mean value of given area
temp = sol(xIdxL:xIdxH, yIdxL:yIdxH);
meanVal = mean(temp(:));

%% find a lower bound
bound_l_ub = meanVal;
bound_l_lb = 0;
bound_l = 0;

% bililve strategy to find the lower bound of the credible region
while (bound_l_ub - bound_l_lb) > stepSize
    bound_l = (bound_l_ub + bound_l_lb)/2;
    dummy(xIdxL:xIdxH, yIdxL:yIdxH) = bound_l;
    energy_dummy = energyG(dummy); 
    
    if energy_dummy <= gamma_alpha_min;
    	bound_l_ub = bound_l;
    else
        bound_l_lb = bound_l;
    end
end

bound_l = (bound_l_ub + bound_l_lb)/2;


%% find an upper bound
bound_h_ub = 1;
bound_h_lb = meanVal;
bound_h = 1;

% bililve strategy to find the upper bound of the credible region
while (bound_h_ub - bound_h_lb) > stepSize
    bound_h = (bound_h_ub + bound_h_lb)/2;
    dummy(xIdxL:xIdxH, yIdxL:yIdxH) = bound_h;
    energy_dummy = energyG(dummy); 
    
    if energy_dummy <= gamma_alpha_min;
    	bound_h_lb = bound_h;
    else
        bound_h_ub = bound_h;
    end
end

bound_h = (bound_h_ub + bound_h_lb)/2;

