function sol_inp = ImgInp(Psit, Psi, sol, maskTest)
%--------------------------------------------------------------------------
%   This program is to do inpainting accrording to given mask
%    
%   Usage: sol_inp = ImgInp(Psit, Psi, sol, maskTest)
%
%   Inputs: 
%       - Psit: wavelet transform
%       - Psi: inverse wavelet transform
%       - sol: given image
%       - maskTest: given mask
%
%   Outputs: 
%       - sol_inp: inpainting result
% 
%   Code by: Xiaohao Cai
%   Last updated: 02/10/2016 
%--------------------------------------------------------------------------

soft = @(z, T) sign(z).*max(abs(z)-T, 0);

% set parameters
maxIte = 200;
lambda = 0.001;

[xLen,yLen] = size(sol);
maskAll=zeros(xLen,yLen);
% generate mask
for i=1:length(maskTest)
    maskAll = maskAll + maskTest{i};
end

maskAll = double(maskAll > 0.5);

sol_inp = sol.*(1-maskAll);
% inpainting
for i=1:maxIte
    wcef = Psit(sol_inp);
    wcef = soft(wcef, lambda);
    sol_inp = Psi(wcef).*maskAll + sol.*(1-maskAll);
end
    


