function superPixImg = gridSam(img, gridSize)
%--------------------------------------------------------------------------
%   This program is to transform a given image to super-pixel image
%    
%   Usage: superPixImg = gridSam(img, gridSize)
%
%   Inputs: 
%       - img: given image
%       - gridSize: super-pixel size
%
%   Outputs: 
%       - superPixImg: generated super-pixel image
% 
%   Code by: Xiaohao Cai
%   Last updated: 01/03/2017
%--------------------------------------------------------------------------

[xLen,yLen] = size(img);
gst = gridSize;

xGridLen = ceil(xLen/gst);
yGridLen = ceil(yLen/gst);

superPixImg = zeros(xLen,yLen);
for jj=1:xGridLen
    for kk=1:yGridLen
        xIdxL = (jj-1)*gst+1; xIdxH = jj*gst;
        yIdxL = (kk-1)*gst+1; yIdxH = kk*gst;

        if xIdxH > xLen
            xIdxH = xLen;
            xIdxL = xLen-gst+1;              
        end

        if yIdxH > yLen
            yIdxH = yLen;
            yIdxL = yLen-gst+1;
        end

        temp = img(xIdxL:xIdxH, yIdxL:yIdxH);
        superPixImg(xIdxL:xIdxH, yIdxL:yIdxH) = mean(temp(:));
    end
end


