function pmcmc_hpd_crediable_region(stringname,imgNum,nameMethod,delta)
%--------------------------------------------------------------------------
% Crediable region (accurate) computation by samples generated by PMCMC
% 
%   Code by: Xiaohao Cai
%   Last updated: 01/03/2017
%--------------------------------------------------------------------------

%% Clear workspace
addpath ./cr_fun

%%
% path to save generated data
pathData_ana=['../data/', stringname, '_mat_ana/'];
pathData_syn=['../data/', stringname, '_mat_syn/'];
pathResult=['../result/',stringname,'/'];
if 0 == exist(pathResult)
    mkdir(pathResult);
end

%% load image
% Data contain: samAna.Xtrac - samples; samAna.logPiTrace - energy
% (minus), and samAna.time - computation time
load([pathData_syn, stringname, '_operators.mat']);
load([pathData_syn, stringname, '_data_org.mat']);
load([pathData_syn, stringname, '_samples_', nameMethod,'_syn.mat']); 
load([pathData_ana, stringname, '_samples_', nameMethod,'_ana.mat']);
load([pathData_syn, stringname, '_measure_noise.mat']);
num_ind = size(Ma,1);

nameMethodTemp = nameMethod;
nameMethodTemp(2:end) = lower(nameMethodTemp(2:end));
samAna = eval(['sam',nameMethodTemp,'Ana']);
samSyn = eval(['sam',nameMethodTemp,'Syn']);
%% Operator
% Set the DWT mode to periodization
dwtmode('per');
% Decomposition level of the wavelet transform.
nlevel = 4;
% Daubechies 8 wavelet operator
[C,S] = wavedec2(im,nlevel,'db8');
Psit = @(x) wavedec2(x, nlevel, 'db8');
Psi = @(x) waverec2(x, S, 'db8');

% Energy of the analysis model
logPi_ana = @(sol) - delta*sum(sum(abs(Psit(sol)))) ...
    - norm(y-A(sol),'fro')^2/(2*sigma_noise^2);

%% Scale samples to positive 


%% Crediable region (accurate) computation
% ---- Compute marginal (pixel-wise) confidence intervals - analysis model
alpha = 0.05;   % for 95% confidence level
% compute the lower and upper bound for each pixel
q_ana = quantile(samAna.Xtrace,[alpha/2,1-alpha/2]);     
% ----

% ---- Compute marginal (pixel-wise) confidence intervals - synthesis model
alpha = 0.05;   % for 95% confidence level
% compute the lower and upper bound for each pixel
samNum = size(samSyn.Xtrace,1);
XtraceSyn = zeros([samNum,size(Psi(squeeze(samSyn.Xtrace(1,:,:))))]);
meanXtrace_ana = zeros(size(squeeze(samAna.Xtrace(1,:,:))));
meanXtrace_syn = zeros(size(meanXtrace_ana));
% for j = 1:samNum
%     XtraceSyn(j,:,:) = Psi(squeeze(samSyn.Xtrace(j,:,:)));
% 
%     % compute energy using analysis model with synthesis samples
%     samSyn.logPiTraceAna(j) = logPi_ana(squeeze(XtraceSyn(j,:,:)));
%     
%     % compute energy using analysis model with analysis samples
%     samAna.logPiTraceAna(j) = logPi_ana(squeeze(samAna.Xtrace(j,:,:)));
% 
%     % compute the mean of the samples for synthesis and analysis models
%     meanXtrace_ana = meanXtrace_ana + squeeze(samAna.Xtrace(j,:,:))/samNum;
%     meanXtrace_syn = meanXtrace_syn + squeeze(XtraceSyn(j,:,:))/samNum;
% end
% q_syn = quantile(XtraceSyn,[alpha/2,1-alpha/2]);     
% % ----

load([pathData_syn, stringname, '_', nameMethod, '_local_cr_pixel.mat']);

save([pathData_syn, stringname, '_', nameMethod, '_local_cr_pixel.mat'], 'q_ana', 'q_syn');
save([pathData_syn, stringname, '_', nameMethod, '_sample_mean.mat'], 'meanXtrace_ana', 'meanXtrace_syn');

% % ---- Compute std according to each pixel
% [xLen,yLen] = size(squeeze(samAna.Xtrace(1,:,:)));
% samStd_ana = zeros(xLen,yLen);
% for ii=1:xLen
%     for jj=1:yLen        
%         samStd_ana(ii,jj) = std(samAna.Xtrace(:,ii,jj));
%         samStd_syn(ii,jj) = std(XtraceSyn(:,ii,jj));
%     end
% end
% % ----
% 
% save([pathResult, stringname, '_', nameMethod, '_std_sample.mat'], 'samStd_ana', 'samStd_syn');  
% 
% % ---- Compute thresholds for joint confidence intervals C^*_alpha
% alpha = 0.01:0.01:0.99;
% % compute upper bound using analysis model
% gamma_alpha_hpd_cr_ana = quantile(-samAna.logPiTraceAna,alpha);
% % compute upper bound using synthesis model
% gamma_alpha_hpd_cr_syn = quantile(-samSyn.logPiTraceAna,alpha);  
% % These thresholds are useful for hypothesis testing (e.g., to reject a test image)
% % Note: the procedure to test an image is as follows: generate a test image
% % "xTest" and compute logPi(xTest). If that value is smaller than
% % eta at level 95%, then we can reject at level 95%.
% % ----
% 
% save([pathData_syn, stringname, '_', nameMethod, '_hpd_cr.mat'], 'gamma_alpha_hpd_cr_ana', ...
%     'gamma_alpha_hpd_cr_syn', 'alpha');

%% Show results   
% % % Show the mean of the generated samples - analysis model
% % meanXtrace_ana(meanXtrace_ana<0)=0;
% % if imgNum==2
% %     figure, imagesc(log10(meanXtrace_ana),[-3 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);
% % else
% %     figure, imagesc(log10(meanXtrace_ana),[-2 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);
% % end
% % colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im)); 
% % fname = [pathResult, stringname, '_', nameMethod, '_mean_sample_ana'  ,'.png'] ;
% % print('-r200', '-dpng', fname)
% % 
% % % Show the mean of the generated samples - synthesis model
% % meanXtrace_syn(meanXtrace_syn<0)=0;
% % if imgNum==2
% %     figure, imagesc(log10(meanXtrace_syn),[-3 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
% % else
% %     figure, imagesc(log10(meanXtrace_syn),[-2 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
% % end
% % colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im)); 
% % fname = [pathResult, stringname, '_', nameMethod, '_mean_sample_syn'  ,'.png'] ;
% % print('-r200', '-dpng', fname)
% % 
% % % Show energy according to each sample
% % xSamNum = 1:1:samNum;
% % figure, plot(xSamNum, -samAna.logPiTrace, '-r', xSamNum, -samSyn.logPiTraceAna, '-b');
% % legend({[nameMethod,'-ana'], [nameMethod,'-syn']}, 'FontSize',12);
% % fname = [pathResult, stringname, '_', nameMethod, '_sample_energy'  ,'.png'] ;
% % print('-r200', '-dpng', fname)

% Set scale for different method
if strcmp('MYULA',nameMethod) == 1
    scaleSet = [0,0.01];
    scaleBar = [0.068 0.042];
else
    scaleSet = [0,10^(-3)];
    scaleBar = [0.063 0.03];
end

% Show the length of the marginal intervals - pixel level - analysis model
if imgNum==2
    figure; imagesc(squeeze(diff(q_ana)),scaleSet); axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+scaleBar(1) a(2)+scaleBar(2) 0.015 0.78]);
else
    figure; imagesc(squeeze(diff(q_ana)),scaleSet); axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1) a(2)+0.035 0.015 0.78]);
end
colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)); truesize; 
fname = [pathResult, stringname, '_', nameMethod, '_local_cr_pixel_ana'  ,'.png'] ;
print('-r200', '-dpng', fname)

% Show the length of the marginal intervals - pixel level - synthesis model
if imgNum==2
    figure; imagesc(squeeze(diff(q_syn)),scaleSet); axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+scaleBar(1) a(2)+scaleBar(2) 0.015 0.78]); 
else
    figure; imagesc(squeeze(diff(q_syn)),scaleSet); axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1) a(2)+0.035 0.015 0.78]); 
end
colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)); truesize; 
fname = [pathResult, stringname, '_', nameMethod, '_local_cr_pixel_syn'  ,'.png'] ;
print('-r200', '-dpng', fname)

% % Show the std of the samples accroding to indi-pixel - analysis model
% figure; imagesc(samStd_ana); axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);
% colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)); truesize; 
% fname = [pathResult, stringname, '_', nameMethod, '_sample_std_ana'  ,'.png'] ;
% print('-r200', '-dpng', fname)
% 
% % Show the std of the samples accroding to indi-pixel - synthesis model
% figure; imagesc(samStd_syn); axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
% colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)); truesize; 
% fname = [pathResult, stringname, '_', nameMethod, '_sample_std_syn'  ,'.png'] ;
% print('-r200', '-dpng', fname)

% show computed HPD credible region upper bound
nameMethodTemp = nameMethod;
if 1 == strcmp(nameMethodTemp,'PMALA')
    nameMethodTemp = 'Px-MALA';
end
    
alpha = 0.01:0.01:0.99;
figure, plot(alpha, gamma_alpha_hpd_cr_ana, '-r', alpha, gamma_alpha_hpd_cr_syn, '-b');
legend({[nameMethodTemp,': analysis'], [nameMethodTemp,': synthesis']}, 'FontSize',18); set(gca,'fontsize',20)
fname = [pathResult, stringname, '_', nameMethod, '_cr_gamma_alpha_acc'  ,'.png'] ;
print('-r200', '-dpng', fname)

% % show all the computed credible region upper bound
% alpha = 0.01:0.01:0.99;
% figure, plot(alpha, gamma_alpha_ana, '--r', alpha, gamma_alpha_ana_acc, '-r', ...
%     alpha, gamma_alpha_syn, '--b', alpha, gamma_alpha_syn_acc, '-b');
% legend({'analysis', 'analysis-acc', 'synthesis', 'synthesis-acc'}, 'FontSize',12);
% fname = [pathResult, stringname, '_', nameMethod, '_cr_gamma_alpha_all'  ,'.png'] ;
% print('-r200', '-dpng', fname)


% % check generated samples
% figure,imagesc(squeeze(XtraceSyn(100,:,:))); cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);
% figure,imagesc(squeeze(samAna.Xtrace(100,:,:))); cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);





