function pmcmc_local_crediable_region(stringname,imgNum,nameMethod,delta)
%--------------------------------------------------------------------------
%   Usage: crediable_region_est_ub_main;
%
%   Crediable region estimation from computing an upper bound by 
%           gamma_a := g(x') + N*(tau_a + 1)
%   where x' is the solution of mim g(x), N is the size of x, 
%   tau_a = sqrt(16*log(3/a)/N), and a \in (4exp(-N/3), 1).
% 
%   Other operators: A is the forward measurement operator and At the 
%   associated adjoint operator, Psit is wavelet transform and Psi its 
%   adjoint, and delta is the parameter balancing the L1 and L2 terms.
% 
%   All Rights Reserved  
%   Code by: Xiaohao Cai
%   Last updated: 01/03/2017
%--------------------------------------------------------------------------

%% Define paths
addpath ./cr_fun

%% 
% path to save generated data
pathData_ana=['../data/', stringname, '_mat_ana/'];
pathData_syn=['../data/', stringname, '_mat_syn/'];
pathResult=['../result/',stringname,'/'];
if 0 == exist(pathResult)
    mkdir(pathResult);
end

%% load image 
load([pathData_syn, stringname, '_operators.mat']);
load([pathData_syn, stringname, '_measure_noise.mat']);
load([pathData_syn, stringname, '_measure_org.mat']);
load([pathData_syn, stringname, '_data_org.mat']);
load([pathData_syn, stringname, '_samples_', nameMethod,'_syn.mat']); 
load([pathData_ana, stringname, '_samples_', nameMethod,'_ana.mat']);
load([pathData_syn, stringname, '_', nameMethod, '_hpd_cr.mat']);
load([pathData_syn, stringname, '_', nameMethod, '_sample_mean.mat']);
sol_syn_mcmc=meanXtrace_syn;
sol_ana_mcmc=meanXtrace_ana;
load([pathData_syn, stringname, '_synthesis_result.mat']);
sol_syn_map=sol;
load([pathData_ana, stringname, '_analysis_result.mat']);
sol_ana_map=sol;

nameMethodTemp = nameMethod;
nameMethodTemp(2:end) = lower(nameMethodTemp(2:end));
samAna = eval(['sam',nameMethodTemp,'Ana']);
samSyn = eval(['sam',nameMethodTemp,'Syn']);
%% Define operators
% Set the DWT mode to periodization
dwtmode('per');
% Decomposition level of the wavelet transform.
nlevel = 4;
% Daubechies 8 wavelet operator
[C,S] = wavedec2(im,nlevel,'db8');
Psit = @(x) wavedec2(x, nlevel, 'db8');
Psi = @(x) waverec2(x, S, 'db8');

% Energy (cost) function
energy_ana = @(sol) delta*sum(sum(abs(Psit(sol)))) ...
    + norm(y - A(sol),'fro')^2/(2*sigma_noise^2);
energy_syn = @(sol) delta*sum(sum(abs(sol))) ...
    + norm(y - A(Psi(sol)),'fro')^2/(2*sigma_noise^2);

% obtain hpd credible region
gamma_alpha_ana = gamma_alpha_hpd_cr_ana;
gamma_alpha_syn = gamma_alpha_hpd_cr_syn;

% use bound from analysis model
gamma_alpha_ana_sel = max(gamma_alpha_ana);
gamma_alpha_syn_sel = max(gamma_alpha_syn);

% grid setting
gridSize = [10, 20, 30];
stepSize=0.1./gridSize;

% --- Way 3: find local credible intervals for grid image using grid samples 
alpha = 0.05;
samNum = size(samSyn.Xtrace,1);
for i=1:length(gridSize)
    for j=1:samNum
        samAnaXtrace(j,:,:) = gridSam(squeeze(samAna.Xtrace(j,:,:)),gridSize(i));
        samSynXtrace(j,:,:) = gridSam(Psi(squeeze(samSyn.Xtrace(j,:,:))),gridSize(i));
    end
    q_grid_syn{i} = quantile(samSynXtrace,[alpha/2,1-alpha/2]); 
    q_grid_ana{i} = quantile(samAnaXtrace,[alpha/2,1-alpha/2]); 
end 
% ---

% % --- Way 2: using HPD credible regions by using MAP estimator
% [gridImg_lh_ana_map, gridImg2_lh_ana_map] = FindItv4GridImg(gridSize,sol_ana_map, ...
%     stepSize, energy_ana, gamma_alpha_ana_sel);
% sol_img_syn_t=Psi(sol_syn_map);sol_img_syn_t(sol_img_syn_t<0)=0;
% [gridImg_lh_syn_map, gridImg2_lh_syn_map] = FindItv4GridImg(gridSize,sol_img_syn_t, ...
%     stepSize, energy_ana, gamma_alpha_syn_sel);
% ---

% --- Way 1: find local credible intervals for grid image using HPD credible regions
[gridImg_lh_ana, gridImg2_lh_ana] = FindItv4GridImg(gridSize,sol_ana_mcmc, ...
    stepSize, energy_ana, gamma_alpha_ana_sel);
% [gridImg_lh_syn, gridImg2_lh_syn] = FindItv4GridImg(gridSize,sol_syn_mcmc, ...
%     stepSize, energy_ana, gamma_alpha_syn_sel);
gridImg_lh_syn = 0;

save([pathResult, stringname, '_', nameMethod, '_cr_lc', '.mat'], 'gridImg_lh_ana', 'gridImg_lh_syn');
% ---

% % --- Way 1: test time performance of using bileval
% tic;
% [gridImg_lh_ana, gridImg2_lh_ana] = FindItv4GridImg(gridSize,sol_ana_mcmc, ...
%     stepSize, energy_ana, gamma_alpha_ana_sel);
% time1 = toc

% tic;
% [gridImg_lh_bl_ana, gridImg2_lh_bl_ana] = FindItv4GridImgBiLevel(gridSize,sol_ana_mcmc, ...
%     stepSize, energy_ana, gamma_alpha_ana_sel);
% time2 = toc
% % ---
% time = toc
   
%% show credible area for image accroding given grid
for j=1:length(gridSize)  
    % choose scale for image showing: using HPD credible regions
    if imgNum==1
        scaleL=0; scaleH=0.3;
    elseif imgNum==2
        scaleL=0; scaleH=0.1;          
    elseif imgNum==3
        scaleL=0; scaleH=0.2;
    else 
        scaleL=0; scaleH=0.2;    
    end
    
    % ---- way 1: using HPD credible regions ----
    % for analysis result - using HPD credible regions
    figure, imagesc(gridImg_lh_ana{j}.cr_bound_h - gridImg_lh_ana{j}.cr_bound_l,[scaleL,scaleH]);  
    axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
    truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
    fname = [pathResult, stringname, '_', nameMethod, '_cr_ana_grid_hpd_', num2str(gridSize(j))  ,'.png'] ;
    print('-r200', '-dpng', fname)

%     % for synthesis result - using HPD credible regions
%     figure, imagesc(gridImg_lh_syn{j}.cr_bound_h - gridImg_lh_syn{j}.cr_bound_l,[scaleL,scaleH]); 
%     axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
%     truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
%     fname = [pathResult, stringname, '_', nameMethod, '_cr_syn_grid_hpd_', num2str(gridSize(j))  ,'.png'] ;
%     print('-r200', '-dpng', fname)

%     % ---- way 2: using HPD credible regions and using MAP estimator ----
%     % for analysis result - using HPD credible regions - using MAP estimator
%     figure, imagesc(gridImg_lh_ana_map{j}.cr_bound_h - gridImg_lh_ana_map{j}.cr_bound_l,[scaleL,scaleH]);  
%     axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
%     truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
%     fname = [pathResult, stringname, '_', nameMethod, '_MAP_cr_ana_grid_hpd_', num2str(gridSize(j))  ,'.png'] ;
%     print('-r200', '-dpng', fname)
% 
%     % for synthesis result - using HPD credible regions - using MAP estimator
%     figure, imagesc(gridImg_lh_syn_map{j}.cr_bound_h - gridImg_lh_syn_map{j}.cr_bound_l,[scaleL,scaleH]); 
%     axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
%     truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
%     fname = [pathResult, stringname, '_', nameMethod, '_MAP_cr_syn_grid_hpd_', num2str(gridSize(j))  ,'.png'] ;
%     print('-r200', '-dpng', fname)
    

%     % choose scale for image showing: using HPD credible regions
%     if imgNum==1
%         scaleL=0; scaleH=10^(-4);
%     elseif imgNum==2
%         scaleL=0; scaleH=10^(-4);          
%     elseif imgNum==3
%         scaleL=0; scaleH=10^(-4);
%     else 
%         scaleL=0; scaleH=10^(-4);    
%     end
%     
%     % ---- way 3: using generated samples ----
%     % for analysis result - using samples
%     figure, imagesc(squeeze(diff(q_grid_ana{j})),[scaleL,scaleH]);  
%     axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
%     truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
%     fname = [pathResult, stringname, '_', nameMethod, '_cr_ana_grid_sam_', num2str(gridSize(j))  ,'.png'] ;
%     print('-r200', '-dpng', fname)
%             
%     % for synthesis result - using samples
%     figure, imagesc(squeeze(diff(q_grid_syn{j})),[scaleL,scaleH]);  
%     axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
%     truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
%     fname = [pathResult, stringname, '_', nameMethod, '_cr_syn_grid_sam_', num2str(gridSize(j))  ,'.png'] ;
%     print('-r200', '-dpng', fname)
end
    

    





















