%% Example_fb_uca
% Example to demonstrate use of unconstraint BPDN models solver.  
% A random Fourier sampling measurement operator is considered.  
% Daubechies 8 wavelets are used for the sparsifying operator.
% 
% Code by: Xiaohao Cai
% Last updated: 16/09/2016

%% current result is not good

%% Clear workspace
clc; clear; close all;

%% Define paths
addpath ../data
addpath misc/
addpath prox_operators/

%% Define parameters
% Coverages (half the plane for Fourier sampling)
p = 0.50;

% Noise level (on the measurements)
input_snr = 30;


%% Read image
stringname='M31.fits';
data = fitsread(stringname);
data=flipud(data);
im=im2double(data);

% Normalise
im = im-min(im(:));
im = im/max(im(:));

% Enforce positivity
im(im<0) = 0;


%% Define operators
% Set the DWT mode to periodization
dwtmode('per');

% Decomposition level of the wavelet transform.
nlevel = 4;

% Daubechies 8 wavelet operator
[C,S] = wavedec2(im,nlevel,'db8');
Psit = @(x) wavedec2(x, nlevel, 'db8');
Psi = @(x) waverec2(x, S, 'db8');

%  Define mask - Uniform sampling of the half Fourier plane
mask = rand(size(im)) < p; % generate a mask with 100*p/% number of pixels
mask(:,1:floor(size(im,2)/2))=0; % set the first half to zero
mask = ifftshift(mask); % swap
mask(1,1)=0; 
mask(floor(size(im,1)/2):end,1)=0;

ind = find(mask==1);
num_ind = numel(ind);
Ma = sparse(1:num_ind, ind, ones(num_ind,1), num_ind, numel(im));

% Composition (Masking o Fourier)
A = @(x) Ma*reshape(fft2(x)/sqrt(num_ind), numel(x), 1);
At = @(x) (ifft2(reshape(Ma'*x(:), size(im))*sqrt(num_ind)));


%% generate measurements
% Apply measurement operator
y = A(im);

% Add Gaussian i.i.d. noise
sigma_noise = 10^(-input_snr/20)*max(im(:));
noise = (randn(size(y)) + 1i*randn(size(y)))*sigma_noise;
y = y + noise;

% get a dirty image by At directly
dirty_im = abs(At(y)); % get an approximation using IFFT to compare


%% Run simulations
% Parameters for BPDN
param1.verbose = 1; % Print log or not
param1.rel_obj = 1e-4; % Stopping criterion for the L1 problem
param1.max_iter = 100; % Max. number of iterations for the L1 problem
param1.tight_L1 = 1; % * Indicate if Psit is a tight frame (1) or not (0)
param1.max_iter_L1 = 100;
param1.rel_obj_L1 = 1e-3;
param1.gamma = 1e-1; % parameter used for soft-threshoulding in L1 projection
param1.verbose_L1 = 0;

delta = 1e1; % Regularisation parameter - need to tune to obtain good result

% Solve BPDN problem with positivity constraint
sol1 = sopt_admm_uca(y, delta, A, At, Psi, Psit, param1);

% Compute SNR
RSNR1 = 20*log10(norm(im,'fro')/norm(im-sol1,'fro'));


%% Show results
figure, imagesc(im),axis off; colorbar
title(['Original image']); colormap gray

figure, imagesc(dirty_im),axis off; colorbar
title(['Solution by IFFT']); colormap gray

figure, imagesc(sol1),axis off; colorbar
title(['SNR=',num2str(RSNR1), 'dB']); colormap gray

