%% Example_fb_uca
% Example to demonstrate use of unconstraint BPDN models solver.  
% A random Fourier sampling measurement operator is considered.  
% Daubechies 8 wavelets are used for the sparsifying operator.
% 
% Code by: Xiaohao Cai
% Last updated: 16/09/2016

%% Define paths
addpath ../data
addpath misc/
addpath prox_operators/

%% Clear workspace
clc; clear; close all;

% data name
stringname = {'M31', 'CYN', 'W28', '3c288'};

for i=1:4
    % path to save generated data
    pathData=['../data/', stringname{i}, '_mat_ana/'];
    pathData_syn=['../data/', stringname{i}, '_mat_syn/'];
    if 0 == exist(pathData)
        mkdir(pathData);
    end

    %% Define parameters
    % Coverages (half the plane for Fourier sampling)
    p = 0.50;

    % Noise level (on the measurements)
    input_snr = 30;


    %% Read image
    data = fitsread([stringname{i},'.fits']);
    data=flipud(data);
    im=im2double(data);
    
    % reduce image size 
    if i==2
        im=imresize(im,[256, 512]);
    end
    
    if i==3 || i==4
        im=imresize(im,[256, 256]);
    end  

    % Normalise
    im = im-min(im(:));
    im = im/max(im(:));

    % Enforce positivity
    im(im<0) = 0;
 
    if i==3
        im(im<0.2) = 0;      
    end
 
    N = numel(im(:));

    %% Define operators
    % Set the DWT mode to periodization
    dwtmode('per');

    % Decomposition level of the wavelet transform.
    nlevel = 4;

    % Daubechies 8 wavelet operator
    [C,S] = wavedec2(im,nlevel,'db8');
    Psit = @(x) wavedec2(x, nlevel, 'db8');
    Psi = @(x) waverec2(x, S, 'db8');

% %     % Define mask
% %     mask=sopt_vdsmask(size(im,1),size(im,2),p);
% %     ind = find(mask==1);
% %     num_ind = numel(ind);
% %     % Masking matrix (sparse matrix in matlab)
% %     Ma = sparse(1:numel(ind), ind, ones(numel(ind), 1), numel(ind), numel(im));
% %     mask = ifftshift(mask); % swap
% % 
% %     N = numel(im(:));
% %     % Composition (Masking o Fourier)
% %     A = @(x) Ma*reshape(fft2(x)/sqrt(N), numel(x), 1);
% %     At = @(x) (ifft2(reshape(Ma'*x(:), size(im))*sqrt(N)));


    %% generate measurements
%     % Apply measurement operator
%     y = A(im);
% 
%     % Add Gaussian i.i.d. noise
%     sigma_noise = 10^(-input_snr/20)*max(im(:));
%     noise = (randn(size(y)) + 1i*randn(size(y)))*sigma_noise;
%     y = y + noise;
    
    %% load image 
    load([pathData_syn, stringname{i}, '_operators.mat']);
    load([pathData_syn, stringname{i}, '_data_org.mat']);
    load([pathData_syn, stringname{i}, '_measure_noise.mat']);

    % get a dirty image by At directly
    dirty_im = abs(At(y)); % get an approximation using IFFT to compare

    %% Run simulations
    % Parameters for BPDN
    param1.verbose = 1; % Print log or not
    param1.rel_obj = 1e-4; % Stopping criterion for the L1 problem
    param1.max_iter = 500; % Max. number of iterations for the L1 problem
    param1.tight_L1 = 1; % * Indicate if Psit is a tight frame (1) or not (0)
    param1.max_iter_L1 = 100;
    param1.rel_obj_L1 = 1e-3;
    param1.gamma = 0.5; % step size in fb algorithm
    param1.verbose_L1 = 0;
%     param1.initsol = zeros(size(At(y)));


    delta = 1e4*2*sigma_noise^2; % Regularisation parameter - need to tune to obtain good result

    tic;
    % Solve BPDN problem with positivity constraint
    sol = sopt_fb_uca(y, delta, A, At, Psi, Psit, param1);
    time = toc/60

    % Compute SNR
    RSNR1 = 20*log10(norm(im,'fro')/norm(im-sol,'fro'));
    
    save([pathData, stringname{i}, '_analysis_result.mat'], 'sol');
end

%% Show results
figure, imagesc(im),axis off; colorbar
title(['Original image']); colormap gray

figure, imagesc(dirty_im),axis off; colorbar
title(['Solution by IFFT']); colormap gray

figure, imagesc(sol),axis off; colorbar
title(['SNR=',num2str(RSNR1), 'dB']); colormap gray

figure, imagesc(log10(dirty_im),[-2 0]); colorbar

sol(sol<0)=0;
figure, imagesc(log10(sol),[-2 0]); colorbar
axis off; colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im))

sol(sol<0)=0;
figure, imagesc(log10(sol),[-3.5 0]); colorbar
axis off; colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im))
