%% Example_fb_ucs
% Example to demonstrate use of unconstraint BPDN models solver.  
% A random Fourier sampling measurement operator is considered.  
% Daubechies 8 wavelets are used for the sparsifying operator.
% 
% Code by: Xiaohao Cai
% Last updated: 16/09/2016

%% Define paths
addpath ../data
addpath ../_crediable_region_compute/cr_fun
addpath misc/
addpath prox_operators/

%% Clear workspace
clc; clear; close all; 
    
% data name
stringname = {'M31', 'CYN', 'W28', '3c288'};

for i=1:4
    % path to save generated data
    pathData=['../data/', stringname{i}, '_mat_syn/'];
    pathData_syn=['../data/', stringname{i}, '_mat_syn/'];
    if 0 == exist(pathData)
        mkdir(pathData);
    end

    %% Define parameters
    % Coverages (half the plane for Fourier sampling)
    p = 0.1;

    % Noise level (on the measurements)
    input_snr = 30;

    %% Read image
    data = fitsread([stringname{i},'.fits']);
    data=flipud(data);
    im=im2double(data);
    
    % reduce image size 
    if i==2
        im=imresize(im,[256, 512]);
    end
    
    if i==3 || i==4
        im=imresize(im,[256, 256]);
    end     
        
    % Normalise
    im = im-min(im(:));
    im = im/max(im(:));

    % Enforce positivity
    im(im<0) = 0;
    
    if i==2
        th = 0.004;
        im(im<th) = 0;  % 0.04    
    end

    if i==3
        th = 0.022;
        im(im<th) = 0;  % 0.04    
    end
    
    if i==4
        th = 0.035;
        im(im<th) = 0;      
    end
    
%     save([pathData, stringname{i}, '_data_org.mat'], 'im'); 
    
    %% Define operators
    % Set the DWT mode to periodization
    dwtmode('per');

    % Decomposition level of the wavelet transform.
    nlevel = 4;

    % Daubechies 8 wavelet operator
    [C,S] = wavedec2(im,nlevel,'db8');
    Psit = @(x) wavedec2(x, nlevel, 'db8');
    Psi = @(x) waverec2(x, S, 'db8');

% %     % Define mask
% %     mask=sopt_vdsmask(size(im,1),size(im,2),p);
% %     ind = find(mask==1);
% %     num_ind = numel(ind);
% %     % Masking matrix (sparse matrix in matlab)
% %     Ma = sparse(1:numel(ind), ind, ones(numel(ind), 1), numel(ind), numel(im));
% %     mask = ifftshift(mask); % swap
% % 
% %     N = numel(im(:));
% %     % Composition (Masking o Fourier)
% %     A = @(x) Ma*reshape(fft2(x)/sqrt(N), numel(x), 1);
% %     At = @(x) (ifft2(reshape(Ma'*x(:), size(im))*sqrt(N)));
% % 
% % %     save([pathData, stringname{i}, '_operators.mat'], 'Ma', 'A', 'At');

    %% generate measurements
% %     % Apply measurement operator
% %     y = A(im);
% %     y_org=y;
% % 
% % %     save([pathData, stringname{i}, '_measure_org.mat'], 'y_org');
% %     
% %     % Add Gaussian i.i.d. noise
% %     sigma_noise = 10^(-input_snr/20)*norm(im(:))/sqrt(N);
% %     noise = (randn(size(y)) + 1i*randn(size(y)))*sigma_noise;
% %     y = y + noise;
% %     
% % %     save([pathData, stringname{i}, '_measure_noise.mat'], 'y', 'sigma_noise');

    %% load image 
    load([pathData_syn, stringname{i}, '_operators.mat']);
    load([pathData_syn, stringname{i}, '_data_org.mat']);
    load([pathData_syn, stringname{i}, '_measure_noise.mat']);

    % get a dirty image by At directly
    dirty_im = abs(At(y)); % get an approximation using IFFT to compare
    
    %% Run simulations
    % Parameters for BPDN
    param1.verbose = 1; % Print log or not
    param1.rel_obj = 1e-4; % Stopping criterion for the L1 problem
    param1.max_iter = 500; % Max. number of iterations for the L1 problem
    param1.gamma = 0.5; % step size in fb algorithm

    delta = 1e4*2*sigma_noise^2; % Regularisation parameter - need to tune to obtain good result

    tic;
    % Solve BPDN problem with positivity constraint
    sol = sopt_fb_ucs(y, delta, A, At, Psi, Psit, param1);
    time = toc/60

    % Compute SNR
    RSNR1 = 20*log10(norm(im,'fro')/norm(im-Psi(sol),'fro'));
    
    save([pathData, stringname{i}, '_synthesis_result.mat'], 'sol');
end

%% Show results
% figure, imagesc(log10(im),[-3.5 0]); colorbar
% axis off; colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im))
% title(['Original image']); 
% 
% % figure, imagesc(Psi(sol)),axis off; colorbar
% % title(['SNR=',num2str(RSNR1), 'dB']);  
% 
% % figure, imagesc(log10(dirty_im),[-2 0]); colorbar
% 
% sol_img_syn = Psi(sol); sol_img_syn(sol_img_syn<0)=0;
% figure, imagesc(log10(sol_img_syn),[-3.5 0]); colorbar
% title(['th = ', num2str(th)]);
% axis off; colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im))
% 
% sol_img_syn = Psi(sol); sol_img_syn(sol_img_syn<0)=0;
% figure, imagesc(log10(sol_img_syn),[-2.5 0]); colorbar
% title(['th = ', num2str(th)]);
% axis off; colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im))

